#ifndef lint
static char rcsid[] = "$Id: play.c,v 1.7 1997/03/25 15:21:14 tommy Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <err.h>
#include <signal.h>
#include <sys/time.h>

#include "play.h"

#define usage_message \
"usage: %s [-Vbdikrstvxh?] filename\n\
    options: \n\
\t-V         verbose mode\n\
\t-b .level  set bass level to .level\n\
\t-d .device set audio device to .device\n\
\t-s .speed  set DSP speed to .speed\n\
\t-t .level  set treble level to .level\n\
\t-v .volume set volume to .volume\n\
\t-x         show extra messages\n\
\t-h or -?   display this message\n\
"

#define usage() \
{ \
    fprintf(stderr, usage_message, myname); \
}

char *myname;

/* defaults */
int  f_verbose = FALSE;
int  f_extra   = FALSE;
int  f_hasdsp  = TRUE;

int intr = FALSE;

int main(int argc, char *argv[])
{
    int c;
    arg_params_t params;

    signal(SIGINT, SIG_IGN);
    
    init_params(&params);

    myname = *argv;

    while ((c = getopt(argc, argv, "Vb:d:s:t:v:xh?")) != EOF) {
	switch (c) {
	case 'V':
	    f_verbose = TRUE;
	    break;
	case 'b':
	    params.bass   = check_numarg(optarg, 'b');
	    break;
	case 'd':
	    params.device = optarg;
	    break;
	case 's':
	    params.speed  = check_numarg(optarg, 's');
	    break;
	case 't':
	    params.treble = check_numarg(optarg, 't');
	    break;
	case 'v':
	    params.volume = check_numarg(optarg, 'v');
	    break;
	case 'x':
	    f_extra = TRUE;
	    break;
	case 'h':
	case '?':
	    usage();
	    exit(0);
	}
    }
    argc -= optind;
    argv += optind;

    if (argc < 1) {
	usage();
    } else {
	char **argvp = argv;

	while (*argvp) {
	    if (f_verbose)
		fprintf(stderr, "%s\n", *argvp);
	    play_audio(*argvp, &params);
	    argvp++;
	}
    }
    return 0;
}
